$ErrorActionPreference = "Stop"

$logDateTimePattern = "[ yyyy-MM-dd HH:mm:ss ] "

Write-Host -Object $(
    $(Get-Date -Format $logDateTimePattern) + "[ INFO ] Execution started."
) -ForegroundColor "Gray"

$startTime = $(Get-Date)

$cmdletParameters = @{
    # "Uri" = "https://api.ipify.org" # temporary ban can happen based on too many requests
    "Uri" = "https://jsonplaceholder.typicode.com/todos/1"
    "TimeoutSec" = 3
    "DisableKeepAlive" = $True
}

$executionCount = 0
$errorCount = 0

$consecutiveErrorCountThreshold  = 10
$errorReportingThreshold = 0

While ( $errorCount -lt $consecutiveErrorCountThreshold ) {
    try
        {
            $executionCount++

            if ( $executionCount % 5000 -eq 0 ) {
                [System.GC]::Collect()
            }

            [System.Void]$(Invoke-RestMethod @cmdletParameters -ErrorAction "Stop")

            [System.Console]::Write(
                "`r" + "Remote calls executed: " + $executionCount +
                $(" " * $($Host.UI.RawUI.WindowSize.Width / 2)) + "`r"
            )

            $errorCount = 0
        }
    catch
        {
            if ( -Not $($_.Exception.Message -Match "Too many requests") ) {
                $errorCount++
            }

            if ( $errorCount -gt $errorReportingThreshold ) {
                $($_.Exception.Message -Split [System.Environment]::NewLine).ForEach({
                    Write-Host -Object $(
                        $(Get-Date -Format $logDateTimePattern) + "[ ERROR ] " + $_
                    ) -ForegroundColor "Red"
                })
            }

            [System.Console]::Write(
                "`r" + "Remote calls executed: " + $executionCount +
                $(" " * $($Host.UI.RawUI.WindowSize.Width / 2)) + "`r"
            )
        }
}

[System.Console]::Write("")

$elapsedTime = $($(Get-Date) - $startTime)

Write-Host -Object $(
    $(Get-Date -Format $logDateTimePattern) + "[ INFO ] Time elapsed: " +
    $("{0:HH:mm:ss}" -f [System.Datetime]$elapsedTime.Ticks)
) -ForegroundColor "Gray"

Write-Host -Object $(
    $(Get-Date -Format $logDateTimePattern) + "[ INFO ] Execution completed."
) -ForegroundColor "Gray"
